/** @file
EasyVR library for STM32F0 Discovery v0.1
Copyright (C) 2014 Maja Filakovic

*/

#include "stm32f0xx.h"

typedef int bool;

#define EVRUsart USART1


/************************************************/

#ifndef EASYVR_RX_TIMEOUT
#define EASYVR_RX_TIMEOUT  100  // default receive timeout (in ms)
#endif

#ifndef EASYVR_WAKE_TIMEOUT
#define EASYVR_WAKE_TIMEOUT  200  // wakeup max delay (in ms)
#endif

#ifndef EASYVR_PLAY_TIMEOUT
#define EASYVR_PLAY_TIMEOUT  5000  // playback max duration (in ms)
#endif

#ifndef EASYVR_TOKEN_TIMEOUT
#define EASYVR_TOKEN_TIMEOUT  1500  // token max duration (in ms)
#endif

/**
  An implementation of the %EasyVR communication protocol.
	*/

	
enum {
		NO_TIMEOUT = 0, INFINITE = -1,
    DEF_TIMEOUT = EASYVR_RX_TIMEOUT,
    WAKE_TIMEOUT = EASYVR_WAKE_TIMEOUT,
    PLAY_TIMEOUT = EASYVR_PLAY_TIMEOUT,
    TOKEN_TIMEOUT = EASYVR_TOKEN_TIMEOUT,
    RESET_TIMEOUT = 40000,
	};

void EVRSend(uint8_t c);
void EVRSendCmd(uint8_t c);
void EVRSendArg(int8_t c);
void EVRSendGroup(uint8_t c);
int EVRRecv(int16_t timeout);
bool EVRRecvArg(int8_t *c, int16_t timeout); //!!!

//
/** Module identification number (firmware version) */
//
typedef enum {
	VRBOT,    /**< Identifies a VRbot module */
	EASYVR,   /**< Identifies an EasyVR module */
	EASYVR2,  /**< Identifies an EasyVR module version 2 */
} evrModuleId_t;

//
/** Language to use for recognition of built-in words */
//
typedef enum {
	ENGLISH = 0,  /**< Uses the US English word sets */
  ITALIAN = 1,  /**< Uses the Italian word sets */
  JAPANESE = 2, /**< Uses the Japanese word sets */
  GERMAN = 3,   /**< Uses the German word sets */
  SPANISH = 4,  /**< Uses the Spanish word sets */
  FRENCH = 5,   /**< Uses the French word sets */
} evrLanguage_t;

//
/** Special group numbers for recognition of custom commands */
//
typedef enum {
  TRIGGER = 0,    /**< The trigger group (shared with built-in trigger word) */
  PASSWORD = 16,  /**< The password group (uses speaker verification technology) */
} evrGroup_t;

//
/** Index of built-in word sets */
//
typedef enum {
  TRIGGER_SET,    /**< The built-in trigger word set */
  ACTION_SET,     /**< The built-in action word set */
  DIRECTION_SET,  /**< The built-in direction word set */
  NUMBER_SET,     /**< The built-in number word set */
} evrWordset_t;

//
/** Confidence thresholds for the knob settings,
  used for recognition of built-in words or custom grammars
  (not used for the mixed trigger group) */
//
typedef enum {
  LOOSER = 0,   /**< Lowest threshold, most results reported */
  LOOSE = 1,    /**< Lower threshold, more results reported */
  TYPICAL = 2,  /**< Typical threshold (deafult) */
  STRICT = 3,   /**< Higher threshold, fewer results reported */
  STRICTER = 4, /**< Highest threshold, fewest results reported */
} evrKnob_t;

//
  /** Strictness values for the level settings,
  used for recognition of custom commands
  (not used for the mixed trigger group) */
//
typedef enum {
  EASY = 1, /**< Lowest value, most results reported */
  NORMAL = 2,   /**< Typical value (default) */
  HARD = 3,     /**< Slightly higher value, fewer results reported */
  HARDER = 4,   /**< Higher value, fewer results reported */
  HARDEST = 5,  /**< Highest value, fewest results reported */
} evrLevel_t;

//
/** Constants to use for baudrate settings */
//
typedef enum {
  B115200 = 1,  /**< 115200 bps */
  B57600 = 2,   /**< 57600 bps */
  B38400 = 3,   /**< 38400 bps */
  B19200 = 6,   /**< 19200 bps */
  B9600 = 12,   /**< 9600 bps (default) */
} evrBaudrate_t;

//
/** Constants for choosing wake-up method in sleep mode */
//
typedef enum  {
  WAKE_ON_CHAR = 0,       /**< Wake up on any character received */
  WAKE_ON_WHISTLE = 1,    /**< Wake up on whistle or any character received */
  WAKE_ON_LOUDSOUND = 2,  /**< Wake up on a loud sound or any character received */
  WAKE_ON_2CLAPS = 3,     /**< Wake up on double hands-clap or any character received */
  WAKE_ON_3CLAPS = 6,     /**< Wake up on triple hands-clap or any character received */
} evrWakeMode_t;

//
/** Hands-clap sensitivity for wakeup from sleep mode.
  Use in combination with #WAKE_ON_2CLAPS or #WAKE_ON_3CLAPS */
//
typedef enum {
  CLAP_SENSE_LOW = 0,      /**< Lowest threshold */
  CLAP_SENSE_MID = 1,      /**< Typical threshold */
  CLAP_SENSE_HIGH = 2,     /**< Highest threshold */
} evrClapSense_t;

//
/** Pin configuration options for the extra I/O connector */
//
typedef enum {
  OUTPUT_LOW = 0,     /**< Pin is a low output (0V) */
  OUTPUT_HIGH = 1,    /**< Pin is a high output (3V) */
  INPUT_HIZ = 2,      /**< Pin is an high impedance input */
  INPUT_STRONG = 3,   /**< Pin is an input with strong pull-up (~10K) */
  INPUT_WEAK = 4,     /**< Pin is an input with weak pull-up (~200K) */
} evrPinConfig_t;

//
/** Available pin numbers on the extra I/O connector */
//
typedef enum {
  IO1 = 1,  /**< Pin IO1 */
  IO2 = 2,  /**< Pin IO2 */
  IO3 = 3,  /**< Pin IO3 */
} evrPinNumber_t;

//
  /** Some quick volume settings for the sound playback functions
  (any value in the range 0-31 can be used) */
//
typedef enum {
    VOL_MIN = 0,      /**< Lowest volume (almost mute) */
    VOL_HALF = 7,     /**< Half scale volume (softer) */
    VOL_FULL = 15,    /**< Full scale volume (normal) */
    VOL_DOUBLE = 31,  /**< Double gain volume (louder) */
} evrSoundVolume_t;

//
/** Special sound index values, always available even when no soundtable is present */
//
typedef enum {
  BEEP = 0,   /**< Beep sound */
} evrSoundIndex_t;

//
/** Flags used by custom grammars */
//
typedef enum {
  GF_TRIGGER = 0x10,  /**< A bit mask that indicate grammar is a trigger (opposed to commands) */
} evrGrammarFlag_t;

//
/** Noise rejection level for SonicNet token detection (higher value, fewer results) */
//
typedef enum {
  REJECTION_MIN,    /**< Lowest noise rejection, highest sensitivity */
  REJECTION_AVG,    /**< Medium noise rejection, medium sensitivity */
  REJECTION_MAX,    /**< Highest noise rejection, lowest sensitivity */
} evrRejectionLevel_t;

//
/** Error codes used by various functions */
//
typedef enum {
  //-- 0x: Data collection errors (patgen, wordspot, t2si)
  ERR_DATACOL_TOO_LONG        = 0x02, /**< too long (memory overflow) */
  ERR_DATACOL_TOO_NOISY       = 0x03, /**< too noisy */
  ERR_DATACOL_TOO_SOFT        = 0x04, /**< spoke too soft */
  ERR_DATACOL_TOO_LOUD        = 0x05, /**< spoke too loud */
  ERR_DATACOL_TOO_SOON        = 0x06, /**< spoke too soon */
  ERR_DATACOL_TOO_CHOPPY      = 0x07, /**< too many segments/too complex */
  ERR_DATACOL_BAD_WEIGHTS     = 0x08, /**< invalid SI weights */
  ERR_DATACOL_BAD_SETUP       = 0x09, /**< invalid setup */
  
	//-- 1x: Recognition errors (si, sd, sv, train, t2si)
  ERR_RECOG_FAIL              = 0x11, /**< recognition failed */
  ERR_RECOG_LOW_CONF          = 0x12, /**< recognition result doubtful */
  ERR_RECOG_MID_CONF          = 0x13, /**< recognition result maybe */
  ERR_RECOG_BAD_TEMPLATE      = 0x14, /**< invalid SD/SV template */
  ERR_RECOG_BAD_WEIGHTS       = 0x15, /**< invalid SI weights */
  ERR_RECOG_DURATION          = 0x17, /**< incompatible pattern durations */

//-- 2x: T2si errors (t2si)
  ERR_T2SI_EXCESS_STATES      = 0x21, /**< state structure is too big */
  ERR_T2SI_BAD_VERSION        = 0x22, /**< RSC code version/Grammar ROM dont match */
  ERR_T2SI_OUT_OF_RAM         = 0x23, /**< reached limit of available RAM */
  ERR_T2SI_UNEXPECTED         = 0x24, /**< an unexpected error occurred */
  ERR_T2SI_OVERFLOW           = 0x25, /**< ran out of time to process */
  ERR_T2SI_PARAMETER          = 0x26, /**< bad macro or grammar parameter */
  ERR_T2SI_NN_TOO_BIG         = 0x29, /**< layer size out of limits */
  ERR_T2SI_NN_BAD_VERSION     = 0x2A, /**< net structure incompatibility */
  ERR_T2SI_NN_NOT_READY       = 0x2B, /**< initialization not complete */
  ERR_T2SI_NN_BAD_LAYERS      = 0x2C, /**< not correct number of layers */

  ERR_T2SI_TRIG_OOV           = 0x2D, /**< trigger recognized Out Of Vocabulary */
  ERR_T2SI_TOO_SHORT          = 0x2F, /**< utterance was too short */

  //-- 4x: Synthesis errors (talk, sxtalk)
  ERR_SYNTH_BAD_VERSION       = 0x4A, /**< bad release number in speech file */
  ERR_SYNTH_ID_NOT_SET        = 0x4B, /**< (obsolete) bad sentence structure */
  ERR_SYNTH_TOO_MANY_TABLES   = 0x4C, /**< (obsolete) too many talk tables */
  ERR_SYNTH_BAD_SEN           = 0x4D, /**< (obsolete) bad sentence number */
  ERR_SYNTH_BAD_MSG           = 0x4E, /**< bad message data or SX technology files missing */

  //-- 8x: Custom errors
  ERR_CUSTOM_NOTA             = 0x80, /**< none of the above (out of grammar) */

  //-- Cx: Internal errors (all)
  ERR_SW_STACK_OVERFLOW       = 0xC0, /**< no room left in software stack */
  ERR_INTERNAL_T2SI_BAD_SETUP = 0xCC, /**< T2SI test mode error */
} evrErrorCode_t;

//*************************************
/**
    Detects an EasyVR module, waking it from sleep mode and checking
    it responds correctly.
    @retval true if a compatible module has been found
  */
//*************************************
bool EVRdetect(void);

//*************************************
  /**
    Interrupts pending recognition or playback operations.
    @retval true if the request is satisfied and the module is back to ready
  */
//*************************************
bool EVRstop(void);

//*************************************
  /**
    Sets the language to use for recognition of built-in words.
    @param lang (0-5) is one of values in #Language
    @retval true if the operation is successful
  */
//*************************************
bool EVRsetLanguage(int8_t lang);

//*************************************
  /**
    Sets the timeout to use for any recognition task.
    @param seconds (0-31) is the maximum time the module keep listening
    for a word or a command
    @retval true if the operation is successful
  */
//*************************************
bool EVRsetTimeout(int8_t seconds);

//*************************************
  // command management
//*************************************
//--
	
//*************************************
  // command discovery
//*************************************

//*************************************
  /**
    Retrieves the name and training data of a custom command.
    @param group (0-16) is the target group, or one of the values in #Groups
    @param index (0-31) is the index of the command within the selected group
    @param name points to an array of at least 32 characters that holds the
    command label when the function returns
    @param training is a variable that holds the training count when the
    function returns. Additional information about training is available
    through the functions #isConflict() and #getWord() or #getCommand()
    @retval true if the operation is successful
  */
//*************************************
bool EVRdumpCommand(int8_t group, int8_t index, char* name, uint8_t *training);

//*************************************
// custom grammars
//*************************************
//--

//*************************************
// recognition/training
//*************************************
  
//*************************************
	/**
    Starts recognition of a custom command. Results are available after
    #hasFinished() returns true.
    @param group (0-16) is the target group, or one of the values in #Groups
    @note The module is busy until recognition completes and it cannot
    accept other commands. You can interrupt recognition with #stop().
  */
//*************************************
void EVRrecognizeCommand(int8_t group);
  
//*************************************
	/**
    Starts recognition of a built-in word. Results are available after
    #hasFinished() returns true.
    @param wordset (0-3) is the target word set, or one of the values in
    #Wordset, (4-31) is the target custom grammar, if present
    @note The module is busy until recognition completes and it cannot
    accept other commands. You can interrupt recognition with #stop().
  */
//*************************************
void EVRrecognizeWord(int8_t wordset);
 
//*************************************
/**
    Polls the status of on-going recognition, training or asynchronous
    playback tasks.
    @retval true if the operation has completed
  */
//*************************************
bool EVRhasFinished(void);

//*************************************
  // analyse result
//*************************************

//*************************************
  /**
    Gets the recognised command index if any.
    @retval (0-31) is the command index if recognition is successful, (-1) if no
    command has been recognized or an error occurred
  */
//*************************************
int8_t EVRgetCommand(void);

//*************************************
/**
    Gets the recognised word index if any, from built-in sets or custom grammars.
    @retval (0-31) is the command index if recognition is successful, (-1) if no
    built-in word has been recognized or an error occurred
  */
//*************************************
int8_t EVRgetWord(void);
  
//*************************************
	/**
    Gets the last error code if any.
    @retval (0-255) is the error code, (-1) if no error occurred
  */
//*************************************
int16_t EVRgetError(void);
  
//*************************************
	/**
    Retrieves the timeout indicator.
    @retval true if a timeout occurred
  */
//*************************************
bool EVRisTimeout(void);
  
//*************************************	
	// pin I/O functions
//*************************************
  
//*************************************	
	/**
    Configures an I/O pin as an output and sets its value
    @param pin (1-3) is one of values in #PinNumber
    @param pin (0-1) is one of the output values in #PinConfig,
    or Arduino style HIGH and LOW macros
    @retval true if the operation is successful
  */
//*************************************	
bool EVRsetPinOutput(int8_t pin, int8_t value);

//*************************************		
	// sound table functions
//*************************************	
  
//*************************************		
	/**
    Starts playback of a sound from the sound table. Manually check for
    completion with #hasFinished().
    @param index is the index of the target sound in the sound table
    @param volume (0-31) may be one of the values in #SoundVolume
    @note The module is busy until playback completes and it cannot
    accept other commands. You can interrupt playback with #stop().
  */
//*************************************	
void EVRplaySoundAsync(int16_t index, int8_t volume);

//*************************************	
  /**
    Plays a sound from the sound table and waits for completion
    @param index is the index of the target sound in the sound table
    @param volume (0-31) may be one of the values in #SoundVolume
    @retval true if the operation is successful
    @note To alter the maximum time for the wait, define the 
    EASYVR_PLAY_TIMEOUT macro before including the EasyVR library.
  */
//*************************************	
bool EVRplaySound(int16_t index, int8_t volume);
